#include <stdlib.h>

#ifndef HEADERS_H
#define HEADERS_H

struct general_headers {
	/**
	 * Cache-Control: ...
	 *
	 * Used to control the cache
	 *
	 * TODO: implement cache,
	 * for no we will be using "no-cache" for response
	 */
	const char* cache_control;

	/**
	 * Connection: ...
	 *
	 * requests will often have "keep-alive", but since we don't 
	 * support maintain constant connections, every header must have
	 * "close"
	 *
	 * TODO: implmenet constant connections?
	 */
	const char* connection;

	/**
	 * Date: HTTP-date
	 *
	 * must always be included unless the status code is 100 or 101
	 * or 500/503 if its inconvenient/impossible to generate a valid Date
	 */
	const char* date;

	/**
	 * Pragma: ...
	 *
	 * no-cache control directive should be paired with Pragma: "no-cache"
	 */
	const char* pragma;

	/**
	 * Trailer: fieldnames...
	 *
	 * This indicates that there will be a trailing header with the provided
	 * field names
	 *
	 * If there is no trailer, this field should not include anything
	 *
	 * can not include the following header fields:
	 * 	transfer-encoding
	 * 	content-length
	 * 	trailer
	 */
	const char* trailer;

	/**
	 * Transfer-Encoding: ...
	 *
	 * Indicates what type of transformation has been applied to 
	 * the message body
	 */
	const char* transfer_encoding;

	/**
	 * Upgrade: ...
	 *
	 * Allows the client to specify what additional protocols
	 * it supports and would like to use
	 *
	 * Must use this in the 101 (Switching Protocols) response
	 */
	const char* upgrade;

	/**
	 * Via: ...
	 *
	 * MUST be used by gateways and proxies to indicate
	 * intermediate protocols and receipients
	 */
	const char* via;

	/**
	 * Warning: ...
	 *
	 * carries additional info about the status or transformation
	 * of a message, which might not be reflected in the message
	 * 
	 * may carry more than one header TODO: implement this i suppose
	 */
	const char* warning;
};

struct response_headers {
	/**
	 * Accept-Ranges: ...
	 *
	 * If we should accept a range request for a resource, defaults to NONE
	 *
	 * 
	 */
	const char* accept_ranges; 

	/**
	 * Age: ...
	 *
	 * represents time in seconds
	 *
	 * time since the response was generated, this is for cached responses,
	 * if we are cacheing, this header is REQUIRED
	 */
	uint age; 

	/**
	 * ETag: ...
	 *
	 * Current value of the entity tag for the requested tag
	 * only useful if we're sending an entity body+header aswell
	 */
	const char* etag;

	/**
	 * Location: absoluteURI
	 *
	 * For 201 (created), this should be the new source that was created
	 * for 3xx this should indicate the servers preferred URI for automatic redirection
	 */
	const char* location;

	/**
	 * Proxy-Authenticate: ...
	 *
	 * For 407 this is a required field
	 */
	const char* proxy_authenticate;

	/**
	 * Retry-After: (HTTP-date | delta-seconds)
	 *
	 * can be used with a 503 to indicate how long its expeccted to be unavailable
	 * may be used with 3xx to tell how long to wait before redirecting
	 */
	const char* retry_after;

	/**
	 * Server: product/version ...
	 *
	 * Information about the HTTP server
	 */
	const char* server;

	/**
	 * Vary: (* | field-name)
	 *
	 * This is for a cacheable (or non cacheable reponse)
	 * that is subject to server-driven negotiation
	 */
	const char* vary;

	/**
	 * WWW-Authenticate: ...
	 * 
	 * required for 401 
	 */
	const char* www_authenticate;
};

struct request_headers {
	/**
	 * Accept: ...
	 *
	 * Specify all mediatypes which are acceptable for the response
	 *
	 */
	const char* accept;
	
	/**
	 * Accept-Charset: ...
	 *
	 * Sppeicgy the character sets that are acceptable for the response
	 */
	const char* accept_charset;

	/**
	 * Accept-Encoding: ...
	 *
	 * restricts the content-codings that are acceptable in the response
	 */
	const char* accept_encoding;

	/**
	 * Accept-Language: ...
	 *
	 * restricts the set of langauges that are preferred as a response to the request
	 */
	const char* accept_language;

	/**
	 * Authorization: ...
	 *
	 * Used to auth itself with the server
	 */
	const char* authorization;

	/**
	 * Expect: ...
	 *
	 * Used to indicate the server behaviour required by the client
	 *
	 * A server that doeesn't understand/unable to compy, must respond
	 * with the appropriate error status
	 * If the expectations cant be met, respond with 417, otherwise, some 4xx
	 */
	const char* expect;

	/**
	 * From: mailbox
	 *
	 * If given, SHOULD contain an internet email address for the
	 * user that controls the requesting user agent
	 */
	const char* from;

	/**
	 * Host: ...
	 *
	 * Contains the host and port of the resource being requested
	 */
	const char* host;

	/**
	 * If-Match: ...
	 *
	 * used with a method to make it conditional
	 * uses entity tags..
	 */
	const char* if_match;

	/**
	 * If-Modified-Since: ...
	 *
	 * used with a method to make it conditional
	 * uses entity tags..
	 */
	const char* if_modified_since;

	/**
	 * If-None-Match: ...
	 *
	 * used with a method to make it conditional
	 * uses entity tags..
	 */
	const char* if_none_match;

	/**
	 * If-Range: ...
	 *
	 * used with a method to make it conditional
	 * uses entity tags..
	 */
	const char* if_range;

	/**
	 * If-Unmodified-Since: ...
	 *
	 * used with a method to make it conditional
	 * uses entity tags..
	 */
	const char* if_unmodified_since;

	/**
	 * Max-Forwards: ...
	 *
	 * Provided with TRACE and OPTIONS method to limit
	 * the number of proxies or gateways that can forward the request
	 */
	const char* max_forwards;

	/**
	 * Proxy-Authorization: ...
	 *
	 * Allows the client to identify itself to a proxy which requires auth
	 */
	const char* proxy_authorization;

	/**
	 * Range: ...
	 *
	 * Specify range...
	 */
	const char* range;

	/**
	 * Referer: ...
	 *
	 * Specify the address from which the request-uri was obtained
	 */
	const char* referer;

	/**
	 * TE: ...
	 *
	 * what transfer-encodings it is willing to accept in the response
	 * and whether or not it is willing to accept trailer fields 
	 * in a chunked transfer-coding
	 */
	const char* te;
	
	/**
	 * User-Agent: ...
	 *
	 * The useragent of the accessing client
	 *
	 */
	const char* user_agent;
};


#endif
